@@ -14,9 +14,10 @@
 #include "value.h"
 #include "expr.h"
 #include "expr-impl.h"
-#include "datetime.h"
+#include "gnm-datetime.h"
 #include "workbook.h"
 #include "sheet.h"
+#include "number-match.h"
 
 /* ------------------------------------------------------------------------- */
 
@@ -26,11 +27,12 @@ typedef struct {
 	guint         count;
 	CollectFlags  flags;
 	GSList       *info;
-	GnmDateConventions const *date_conv;
+	GODateConventions const *date_conv;
 } collect_floats_t;
 
-static Value *
-callback_function_collect (EvalPos const *ep, Value *value, void *closure)
+static GnmValue *
+callback_function_collect (GnmEvalPos const *ep, GnmValue const *value,
+			   void *closure)
 {
 	gnm_float x;
 	collect_floats_t *cl = (collect_floats_t *)closure;
@@ -68,16 +70,24 @@ callback_function_collect (EvalPos const *ep, Value *value, void *closure)
 			return value_new_error_VALUE (ep);
 		break;
 
-	case VALUE_INTEGER:
 	case VALUE_FLOAT:
 		x = value_get_as_float (value);
 		break;
 
 	case VALUE_STRING:
-	        if (cl->flags & COLLECT_DATES) {
-		        x = datetime_value_to_serial (value, cl->date_conv);
-			if (x == 0)
-			        return value_new_error_VALUE (ep);
+	        if (cl->flags & COLLECT_COERCE_STRINGS) {
+			GnmValue *vc = format_match_number (value_peek_string (value),
+							    NULL,
+							    cl->date_conv);
+			gboolean bad = !vc || VALUE_IS_BOOLEAN (vc);
+			if (vc) {
+				x = value_get_as_float (vc);
+				value_release (vc);
+			} else
+				x = 0;
+
+			if (bad)
+				return value_new_error_VALUE (ep);
 		} else if (cl->flags & COLLECT_IGNORE_STRINGS)
 			goto callback_function_collect_store_info;
 		else if (cl->flags & COLLECT_ZERO_STRINGS)
@@ -121,9 +131,9 @@ callback_function_collect (EvalPos const *ep, Value *value, void *closure)
  * exprlist:       List of expressions to evaluate.
  * cr:             Current location (for resolving relative cells).
  * flags:          COLLECT_IGNORE_STRINGS: silently ignore strings.
+ *                 COLLECT_COERCE_STRINGS: coerce string into numbers
  *                 COLLECT_ZERO_STRINGS: count strings as 0.
  *                   (Alternative: return #VALUE!.)
- *                 COLLECT_DATES: count strings as dates.
  *                 COLLECT_IGNORE_BOOLS: silently ignore bools.
  *                 COLLECT_ZEROONE_BOOLS: count FALSE as 0, TRUE as 1.
  *                   (Alternative: return #VALUE!.)
@@ -141,21 +151,25 @@ callback_function_collect (EvalPos const *ep, Value *value, void *closure)
  * gnm_float.
  */
 static gnm_float *
-collect_floats (GnmExprList *exprlist, EvalPos const *ep, CollectFlags flags,
-		int *n, Value **error, GSList **info)
+collect_floats (int argc, GnmExprConstPtr const *argv,
+		GnmEvalPos const *ep, CollectFlags flags,
+		int *n, GnmValue **error, GSList **info)
 {
-	Value * err;
+	GnmValue * err;
 	collect_floats_t cl;
-	CellIterFlags iter_flags = (flags & COLLECT_IGNORE_BLANKS)
-		? CELL_ITER_IGNORE_BLANK : CELL_ITER_ALL;
-	if (flags & COLLECT_IGNORE_SUBTOTAL)
-		iter_flags |= CELL_ITER_IGNORE_SUBTOTAL;
+	CellIterFlags iter_flags = CELL_ITER_ALL;
 
 	if (info) {
-		flags = flags | COLLECT_INFO;
+		flags |= COLLECT_INFO;
 		*info = NULL;
-	} else
-		flags = flags & COLLECT_NO_INFO_MASK;
+	} else {
+		if (flags & COLLECT_IGNORE_BLANKS)
+			iter_flags = CELL_ITER_IGNORE_BLANK;
+		flags &= ~COLLECT_INFO;
+	}
+
+	if (flags & COLLECT_IGNORE_SUBTOTAL)
+		iter_flags |= CELL_ITER_IGNORE_SUBTOTAL;
 
 	cl.alloc_count = 20;
 	cl.data = g_new (gnm_float, cl.alloc_count);
@@ -164,11 +178,13 @@ collect_floats (GnmExprList *exprlist, EvalPos const *ep, CollectFlags flags,
 	cl.info = NULL;
 	cl.date_conv = workbook_date_conv (ep->sheet->workbook);
 
-	err = function_iterate_argument_values (ep, &callback_function_collect,
-		&cl, exprlist, TRUE, iter_flags);
+	err = function_iterate_argument_values
+		(ep, &callback_function_collect, &cl,
+		 argc, argv,
+		 TRUE, iter_flags);
 
 	if (err) {
-		g_assert (err->type == VALUE_ERROR);
+		g_assert (VALUE_IS_ERROR (err));
 		g_free (cl.data);
 		g_slist_free (cl.info);
 		*error = err;
@@ -186,37 +202,30 @@ collect_floats (GnmExprList *exprlist, EvalPos const *ep, CollectFlags flags,
    Presumably most useful when the value is an array.  */
 
 gnm_float *
-collect_floats_value (Value const *val, EvalPos const *ep,
-		      CollectFlags flags, int *n, Value **error)
+collect_floats_value (GnmValue const *val, GnmEvalPos const *ep,
+		      CollectFlags flags, int *n, GnmValue **error)
 {
-	GnmExprList *exprlist;
-	GnmExprConstant expr_val;
-	gnm_float *res;
+	GnmExpr expr_val;
+	GnmExprConstPtr argv[1] = { &expr_val };
 
-	gnm_expr_constant_init (&expr_val, val);
-	exprlist = gnm_expr_list_prepend (NULL, &expr_val);
-	res = collect_floats (exprlist, ep, flags, n, error, NULL);
-	gnm_expr_list_free (exprlist);
-
-	return res;
+	gnm_expr_constant_init (&expr_val.constant, val);
+	return collect_floats (1, argv, ep, flags, n, error, NULL);
 }
 
 /* ------------------------------------------------------------------------- */
 /* Like collect_floats_value, but keeps info on missing values */
 
-static gnm_float *
-collect_floats_value_with_info (Value const *val, EvalPos const *ep,
+gnm_float *
+collect_floats_value_with_info (GnmValue const *val, GnmEvalPos const *ep,
 				CollectFlags flags, int *n, GSList **info,
-				Value **error)
+				GnmValue **error)
 {
-	GnmExprList *exprlist;
-	GnmExprConstant expr_val;
+	GnmExpr expr_val;
+	GnmExprConstPtr argv[1] = { &expr_val };
 	gnm_float *res;
 
-	gnm_expr_constant_init (&expr_val, val);
-	exprlist = gnm_expr_list_prepend (NULL, &expr_val);
-	res = collect_floats (exprlist, ep, flags, n, error, info);
-	gnm_expr_list_free (exprlist);
+	gnm_expr_constant_init (&expr_val.constant, val);
+	res = collect_floats (1, argv, ep, flags, n, error, info);
 
 	if (info)
 		*info = g_slist_reverse (*info);
@@ -227,17 +236,18 @@ collect_floats_value_with_info (Value const *val, EvalPos const *ep,
 
 /* ------------------------------------------------------------------------- */
 
-Value *
-float_range_function (GnmExprList *exprlist, FunctionEvalInfo *ei,
+GnmValue *
+float_range_function (int argc, GnmExprConstPtr const *argv,
+		      GnmFuncEvalInfo *ei,
 		      float_range_function_t func,
 		      CollectFlags flags,
 		      GnmStdError func_error)
 {
-	Value *error = NULL;
+	GnmValue *error = NULL;
 	gnm_float *vals, res;
 	int n, err;
 
-	vals = collect_floats (exprlist, ei->pos, flags, &n, &error, NULL);
+	vals = collect_floats (argc, argv, ei->pos, flags, &n, &error, NULL);
 	if (!vals)
 		return (error != VALUE_TERMINATE) ? error : NULL;
 
@@ -292,7 +302,7 @@ cb_int_descending (gconstpointer a, gconstpointer b)
  *  @list_2:
  *
  */
-static GSList*
+GSList*
 union_of_int_sets (GSList * list_1, GSList * list_2)
 {
 	GSList *list_res = NULL;
@@ -335,7 +345,7 @@ cb_remove_missing_el (gpointer data, gpointer user_data)
  * Note this implementation returns the same array as passed in!
  * The order of the elements in the list may be changed.
  */
-static GArray*
+GArray*
 strip_missing (GArray * data, GSList **missing)
 {
 	GArray *new_data = data;
@@ -352,16 +362,17 @@ strip_missing (GArray * data, GSList **missing)
 }
 
 
-Value *
-float_range_function2 (Value *val0, Value *val1, FunctionEvalInfo *ei,
+GnmValue *
+float_range_function2 (GnmValue const *val0, GnmValue const *val1,
+		       GnmFuncEvalInfo *ei,
 		       float_range_function2_t func,
 		       CollectFlags flags,
 		       GnmStdError func_error)
 {
 	gnm_float *vals0, *vals1;
 	int n0, n1;
-	Value *error = NULL;
-	Value *res;
+	GnmValue *error = NULL;
+	GnmValue *res;
 	GSList *missing0 = NULL;
 	GSList *missing1 = NULL;
 
@@ -395,6 +406,7 @@ float_range_function2 (Value *val0, Value *val1, FunctionEvalInfo *ei,
 			g_free (vals0);
 			gval = strip_missing (gval, &missing);
 			vals0 = (gnm_float *)gval->data;
+			n0 = gval->len;
 			g_array_free (gval, FALSE);
 
 			gval = g_array_new (FALSE, FALSE, sizeof (gnm_float));
@@ -402,13 +414,18 @@ float_range_function2 (Value *val0, Value *val1, FunctionEvalInfo *ei,
 			g_free (vals1);
 			gval = strip_missing (gval, &missing);
 			vals1 = (gnm_float *)gval->data;
+			n1 = gval->len;
 			g_array_free (gval, FALSE);
 
 			g_slist_free (missing0);
 			g_slist_free (missing1);
 			g_slist_free (missing);
-		}
 
+			if (n0 != n1) {
+				g_warning ("This should not happen.  n0=%d n1=%d\n",
+					   n0, n1);
+			}
+		}
 
 		if (func (vals0, vals1, n0, &fres))
 			res = value_new_error_std (ei->pos, func_error);
@@ -429,11 +446,12 @@ typedef struct {
 	CollectFlags  flags;
 } collect_strings_t;
 
-static Value *
-callback_function_collect_strings (EvalPos const *ep, Value *value, void *closure)
+static GnmValue *
+callback_function_collect_strings (GnmEvalPos const *ep, GnmValue const *value,
+				   void *closure)
 {
 	char *text = NULL;
-	collect_strings_t *cl = (collect_strings_t *)closure;
+	collect_strings_t *cl = closure;
 
 	if (value == NULL) {
 		if (cl->flags & COLLECT_IGNORE_BLANKS)
@@ -442,7 +460,6 @@ callback_function_collect_strings (EvalPos const *ep, Value *value, void *closur
 	} else switch (value->type) {
 	case VALUE_EMPTY:
 	case VALUE_BOOLEAN:
-	case VALUE_INTEGER:
 	case VALUE_FLOAT:
 	case VALUE_STRING:
 		text = value_get_as_string (value);
@@ -460,8 +477,7 @@ callback_function_collect_strings (EvalPos const *ep, Value *value, void *closur
 			return value_new_error_VALUE (ep);
 		break;
 	default:
-		text = g_strdup_printf ("Trouble in callback_function_collect. (%d)",
-					value->type);
+		g_assert_not_reached ();
 		break;
 	}
 
@@ -485,20 +501,23 @@ callback_function_collect_strings (EvalPos const *ep, Value *value, void *closur
  */
 
 static GSList *
-collect_strings (GnmExprList *exprlist, EvalPos const *ep, CollectFlags flags, Value **error)
+collect_strings (int argc, GnmExprConstPtr const *argv,
+		 GnmEvalPos const *ep, CollectFlags flags,
+		 GnmValue **error)
 {
-	Value * err;
+	GnmValue * err;
 	collect_strings_t cl;
 
 	cl.data = NULL;
 	cl.flags = flags;
 
-	err = function_iterate_argument_values (ep, &callback_function_collect_strings,
-		&cl, exprlist,
+	err = function_iterate_argument_values
+		(ep, &callback_function_collect_strings, &cl,
+		 argc, argv,
 		TRUE, (flags & COLLECT_IGNORE_BLANKS) ?  CELL_ITER_IGNORE_BLANK : CELL_ITER_ALL);
 
 	if (err) {
-		g_assert (err->type == VALUE_ERROR);
+		g_assert (VALUE_IS_ERROR (err));
 		g_slist_foreach (cl.data, (GFunc)g_free, NULL);
 		g_slist_free (cl.data);
 		*error = err;
@@ -509,18 +528,19 @@ collect_strings (GnmExprList *exprlist, EvalPos const *ep, CollectFlags flags, V
 }
 
 
-Value *
-string_range_function (GnmExprList *exprlist, FunctionEvalInfo *ei,
+GnmValue *
+string_range_function (int argc, GnmExprConstPtr const *argv,
+		       GnmFuncEvalInfo *ei,
 		       string_range_function_t func,
 		       CollectFlags flags,
 		       GnmStdError func_error)
 {
-	Value *error = NULL;
+	GnmValue *error = NULL;
 	GSList *vals;
 	char *res = NULL;
 	int err;
 
-	vals = collect_strings (exprlist, ei->pos, flags, &error);
+	vals = collect_strings (argc, argv, ei->pos, flags, &error);
 	if (!vals)
 		return (error != VALUE_TERMINATE) ? error : NULL;
 
@@ -529,8 +549,7 @@ string_range_function (GnmExprList *exprlist, FunctionEvalInfo *ei,
 	g_slist_free (vals);
 
 	if (err) {
-		if (res)
-			g_free (res);
+		g_free (res);
 		return value_new_error_std (ei->pos, func_error);
 	} else {
 		return value_new_string_nocopy (res);
