@@ -21,7 +21,7 @@
 
 #include <gnumeric-config.h>
 #include "gnumeric.h"
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include "consolidate.h"
 
 #include "cell.h"
