@@ -20,11 +20,11 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 #include <gnumeric-config.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <gnumeric.h>
 #include "dialogs.h"
 #include "help.h"
@@ -40,7 +40,8 @@
 #include <expr.h>
 #include <expr-impl.h>
 #include <func.h>
-#include <format.h>
+#include <gnm-format.h>
+#include <goffice/utils/go-locale.h>
 #include <widgets/gnumeric-expr-entry.h>
 #include <widgets/gnumeric-cell-renderer-expr-entry.h>
 
@@ -69,6 +70,7 @@ typedef struct
 	GtkWidget *selector_button;
 	GtkWidget *clear_button;
 	GtkWidget *zoom_button;
+	GtkWidget *array_button;
 	GtkWidget *main_button_area;
 	GtkTreePath* active_path;
 	char * prefix;
@@ -85,6 +87,7 @@ typedef struct
 
 	GnumericCellRendererExprEntry *cellrenderer;
 	GtkTreeViewColumn *column;
+	GtkCellEditable *editable;
 } FormulaGuruState;
 
 enum {
@@ -165,7 +168,7 @@ dialog_formula_guru_update_this_parent (GtkTreeIter *parent, FormulaGuruState *s
 			}
 
 			if (not_first) {
-				text = g_string_append_c (text, format_get_arg_sep ());
+				text = g_string_append_c (text, go_locale_get_arg_sep ());
 			}
 
 			if (find_origin && origin != NULL) {
@@ -373,7 +376,11 @@ dialog_formula_guru_load_fd (GtkTreePath *path, GnmFunc const *fd,
 				gtk_tree_path_free (new_path);
 				return;
 			}
- 		}
+ 		} else {
+			gtk_tree_store_clear (state->model);
+			gtk_tree_path_free (new_path);
+			return;
+		}
 		gtk_tree_path_free (new_path);
 	}
 
@@ -444,10 +451,6 @@ dialog_formula_guru_load_expr (GtkTreePath const *parent_path, gint child_num,
 			       GnmExpr const *expr, FormulaGuruState *state)
 {
 	GtkTreePath *path;
-	char *text;
-	GSList *args;
-	GtkTreeIter iter;
-	int i;
 
 	if (parent_path == NULL)
 		path = gtk_tree_path_new_first ();
@@ -457,59 +460,54 @@ dialog_formula_guru_load_expr (GtkTreePath const *parent_path, gint child_num,
 		gtk_tree_path_append_index (path, child_num);
 	}
 
-	switch (expr->any.oper) {
-	case GNM_EXPR_OP_FUNCALL:
+	switch (GNM_EXPR_GET_OPER (expr)) {
+	case GNM_EXPR_OP_FUNCALL: {
+		int i;
+		GtkTreeIter iter;
+
 		dialog_formula_guru_load_fd (path, expr->func.func, state);
-		for (args = expr->func.arg_list, i = 0; args; args = args->next, i++)
+		for (i = 0; i < expr->func.argc; i++)
 			dialog_formula_guru_load_expr (path, i,
-						       (GnmExpr const *) args->data,
+						       expr->func.argv[i],
 						       state);
-		gtk_tree_path_append_index (path, i - 1);
+		gtk_tree_path_append_index (path, MAX (0, i - 1));
 		if (gtk_tree_model_get_iter (GTK_TREE_MODEL (state->model),
                                              &iter, path))
 			dialog_formula_guru_adjust_varargs (&iter, state);
 
 		break;
+	}
+
 	case GNM_EXPR_OP_ANY_BINARY:
 	case GNM_EXPR_OP_UNARY_NEG:
-	default:
-		text = gnm_expr_as_string (expr, state->pos,
-			gnm_expr_conventions_default);
+	default: {
+		char *text = gnm_expr_as_string
+			(expr, state->pos,
+			 sheet_expr_conventions (state->pos->sheet));
 		dialog_formula_guru_load_string (path, text, state);
 		g_free (text);
 		break;
+	}
 
 	}
 	gtk_tree_path_free (path);
 }
 
-
-/**
- * dialog_function_select_destroy:
- * @window:
- * @state:
- *
- * Destroy the dialog and associated data structures.
- *
- **/
 static void
-dialog_formula_guru_destroy (FormulaGuruState *state)
+cb_dialog_formula_guru_destroy (FormulaGuruState *state)
 {
 	wbcg_edit_detach_guru (state->wbcg);
 	wbcg_edit_finish (state->wbcg, WBC_EDIT_REJECT, NULL);
 
+	if (state->model != NULL)
+		g_object_unref (G_OBJECT (state->model));
 	g_free (state->prefix);
-	state->prefix = NULL;
 	g_free (state->suffix);
-	state->suffix = NULL;
 	g_free (state->pos);
-	state->pos = NULL;
-
-	if (state->gui != NULL) {
+	if (state->editable)
+		g_object_unref (state->editable);
+	if (state->gui != NULL)
 		g_object_unref (G_OBJECT (state->gui));
-		state->gui = NULL;
-	}
-	state->dialog = NULL;
 	g_free (state);
 }
 
@@ -626,6 +624,12 @@ cb_dialog_formula_guru_clear_clicked (G_GNUC_UNUSED GtkWidget *button,
 	return;
 }
 
+static gboolean
+dialog_formula_guru_is_array (FormulaGuruState *state)
+{
+	return gtk_toggle_button_get_active 
+		(GTK_TOGGLE_BUTTON (state->array_button));
+}
 
 /**
  * cb_dialog_formula_guru_ok_clicked:
@@ -642,7 +646,10 @@ cb_dialog_formula_guru_ok_clicked (G_GNUC_UNUSED GtkWidget *button,
 		gnumeric_cell_renderer_expr_entry_editing_done (
 			GTK_CELL_EDITABLE (state->cellrenderer->entry),
 			state->cellrenderer);
-	wbcg_edit_finish (state->wbcg, WBC_EDIT_ACCEPT, NULL);
+	wbcg_edit_finish (state->wbcg, 
+			  dialog_formula_guru_is_array (state)
+			  ? WBC_EDIT_ACCEPT_ARRAY
+			  : WBC_EDIT_ACCEPT, NULL);
 }
 
 static void
@@ -681,28 +688,44 @@ cb_dialog_formula_guru_row_collapsed (G_GNUC_UNUSED GtkTreeView *treeview,
 
 static void
 cb_dialog_formula_guru_edited (G_GNUC_UNUSED GtkCellRendererText *cell,
-	gchar               *path_string,
-	gchar               *new_text,
-        FormulaGuruState    *state)
+			       gchar               *path_string,
+			       gchar               *new_text,
+			       FormulaGuruState    *state)
 {
 	GtkTreeIter iter;
 	GtkTreePath *path;
+	gboolean have_iter = FALSE;
 
 	path = gtk_tree_path_new_from_string (path_string);
 
-	gtk_tree_model_get_iter (GTK_TREE_MODEL (state->model), &iter, path);
+	have_iter = gtk_tree_model_get_iter (GTK_TREE_MODEL (state->model), 
+					     &iter, path);
+	gtk_tree_path_free (path);
+	if (!have_iter) 
+		return;
 	gtk_tree_store_set (state->model, &iter, FUN_ARG_ENTRY, new_text, -1);
 
 	if (g_utf8_strlen (new_text, -1) > 0)
 		dialog_formula_guru_adjust_varargs (&iter, state);
 
-	gtk_tree_path_free (path);
 
 	dialog_formula_guru_update_parent (&iter, state, gtk_tree_model_get_path
 					   (GTK_TREE_MODEL (state->model), &iter),
 					   0, g_utf8_strlen (new_text, -1));
 }
 
+static void
+cb_dialog_formula_guru_editing_started (GtkCellRenderer *cell,
+					GtkCellEditable *editable,
+					const gchar *path,
+					FormulaGuruState *state)
+{
+	g_object_ref (editable);
+	if (state->editable)
+		g_object_unref (state->editable);
+	state->editable = editable;
+}
+
 /* Bad bad hack to be removed with Gtk2.2 */
 /* The idea to this code is due to Jonathan Blandford */
 
@@ -715,15 +738,21 @@ typedef struct
 static gboolean
 real_start_editing_cb (IdleData *idle_data)
 {
-  gtk_widget_grab_focus (GTK_WIDGET (idle_data->state->treeview));
-  gtk_tree_view_set_cursor (idle_data->state->treeview,
-			    idle_data->path,
-			    idle_data->state->column,
-			    TRUE);
+	FormulaGuruState *state = idle_data->state;
+	GtkTreePath *path = idle_data->path;
 
-  gtk_tree_path_free (idle_data->path);
-  g_free (idle_data);
-  return FALSE;
+	if (state->editable)
+		gtk_cell_editable_editing_done (state->editable);
+
+	gtk_widget_grab_focus (GTK_WIDGET (state->treeview));
+	gtk_tree_view_set_cursor (state->treeview,
+				  path,
+				  state->column,
+				  TRUE);
+
+	gtk_tree_path_free (path);
+	g_free (idle_data);
+	return FALSE;
 }
 
 static gboolean
@@ -731,41 +760,43 @@ start_editing_cb (GtkTreeView      *tree_view,
 		  GdkEventButton   *event,
 		  FormulaGuruState *state)
 {
-  GtkTreePath *path;
-  GtkTreeIter iter;
+	GtkTreePath *path;
+	GtkTreeIter iter;
 
-  if (event->window != gtk_tree_view_get_bin_window (tree_view))
-    return FALSE;
-  if (state->treeview != tree_view)
-    return FALSE;
+	if (event->window != gtk_tree_view_get_bin_window (tree_view))
+		return FALSE;
+	if (state->treeview != tree_view)
+		return FALSE;
 
-  if (gtk_tree_view_get_path_at_pos (tree_view,
-				     (gint) event->x,
-				     (gint) event->y,
-				     &path, NULL,
-				     NULL, NULL) &&
-      gtk_tree_model_get_iter (GTK_TREE_MODEL (state->model),
-			       &iter, path))
-    {
-      IdleData *idle_data;
-      gboolean is_non_fun;
+	if (gtk_tree_view_get_path_at_pos (tree_view,
+					   (gint) event->x,
+					   (gint) event->y,
+					   &path, NULL,
+					   NULL, NULL) &&
+	    gtk_tree_model_get_iter (GTK_TREE_MODEL (state->model),
+				     &iter, path))
+	{
+		IdleData *idle_data;
+		gboolean is_non_fun;
 
-      gtk_tree_model_get (GTK_TREE_MODEL (state->model), &iter,
-			  IS_NON_FUN, &is_non_fun,
-			  -1);
+		gtk_tree_model_get (GTK_TREE_MODEL (state->model), &iter,
+				    IS_NON_FUN, &is_non_fun,
+				    -1);
 
-      if (!is_non_fun)
-	      return FALSE;
+		if (!is_non_fun) {
+			gtk_tree_path_free (path);
+			return FALSE;
+		}
 
-      idle_data = g_new (IdleData, 1);
-      idle_data->path = path;
-      idle_data->state = state;
+		idle_data = g_new (IdleData, 1);
+		idle_data->path = path;
+		idle_data->state = state;
 
-      g_signal_stop_emission_by_name (G_OBJECT (tree_view), "button_press_event");
-      g_idle_add ((GSourceFunc) real_start_editing_cb, idle_data);
-      return TRUE;
-    }
-  return FALSE;
+		g_signal_stop_emission_by_name (G_OBJECT (tree_view), "button_press_event");
+		g_idle_add ((GSourceFunc) real_start_editing_cb, idle_data);
+		return TRUE;
+	}
+	return FALSE;
 }
 
 /* End of bad bad hack*/
@@ -809,6 +840,9 @@ dialog_formula_guru_init (FormulaGuruState *state)
 	state->cellrenderer = GNUMERIC_CELL_RENDERER_EXPR_ENTRY (renderer);
 	g_signal_connect (G_OBJECT (renderer), "edited",
 			  G_CALLBACK (cb_dialog_formula_guru_edited), state);
+	state->editable = NULL;
+	g_signal_connect (G_OBJECT (renderer), "editing-started",
+			  G_CALLBACK (cb_dialog_formula_guru_editing_started), state);
 	column = gtk_tree_view_column_new_with_attributes (_("Function/Argument"),
 							   renderer,
 							   "text", FUN_ARG_ENTRY,
@@ -825,6 +859,10 @@ dialog_formula_guru_init (FormulaGuruState *state)
 
 	/* Finished set-up of treeview */
 
+	state->array_button = glade_xml_get_widget (state->gui, 
+						    "array_button");
+	gtk_widget_set_sensitive (state->array_button, TRUE);
+
 	state->ok_button = glade_xml_get_widget (state->gui, "ok_button");
 	gtk_widget_set_sensitive (state->ok_button, TRUE);
 	g_signal_connect (G_OBJECT (state->ok_button),
@@ -859,7 +897,7 @@ dialog_formula_guru_init (FormulaGuruState *state)
 		glade_xml_get_widget (state->gui, "help_button"),
 		GNUMERIC_HELP_LINK_FORMULA_GURU);
 	g_object_set_data_full (G_OBJECT (state->dialog),
-		"state", state, (GDestroyNotify) dialog_formula_guru_destroy);
+		"state", state, (GDestroyNotify) cb_dialog_formula_guru_destroy);
 
 	wbcg_edit_attach_guru (state->wbcg, state->dialog);
 
@@ -926,22 +964,22 @@ dialog_formula_guru (WorkbookControlGUI *wbcg, GnmFunc const *fd)
 	}
 
 	/* Get the dialog and check for errors */
-	gui = gnm_glade_xml_new (GNM_CMD_CONTEXT (wbcg),
+	gui = gnm_glade_xml_new (GO_CMD_CONTEXT (wbcg),
 		"formula-guru.glade", NULL, NULL);
 	if (gui == NULL)
 		return;
 
 	state = g_new (FormulaGuruState, 1);
 	state->wbcg  = wbcg;
-	state->wb    = wb_control_workbook (WORKBOOK_CONTROL (wbcg));
+	state->wb    = wb_control_get_workbook (WORKBOOK_CONTROL (wbcg));
 	state->gui   = gui;
 	state->active_path = NULL;
 	state->pos = NULL;
 
 	sv = wb_control_cur_sheet_view (WORKBOOK_CONTROL (wbcg));
 	cell = sheet_cell_get (sv_sheet (sv), sv->edit_pos.col, sv->edit_pos.row);
-	if (cell != NULL && cell_has_expr (cell))
-		expr = gnm_expr_first_func (cell->base.expression);
+	if (cell != NULL && gnm_cell_has_expr (cell))
+		expr = gnm_expr_top_first_funcall (cell->base.texpr);
 
 	if (expr == NULL) {
 		wbcg_edit_start (wbcg, TRUE, TRUE);
@@ -953,9 +991,10 @@ dialog_formula_guru (WorkbookControlGUI *wbcg, GnmFunc const *fd)
 		char *func_str;
 
 		state->pos = g_new (GnmParsePos, 1);
-		func_str = gnm_expr_as_string (expr,
-			parse_pos_init_cell (state->pos, cell),
-			gnm_expr_conventions_default);
+		func_str = gnm_expr_as_string
+			(expr,
+			 parse_pos_init_cell (state->pos, cell),
+			 sheet_expr_conventions (sv_sheet (sv)));
 
 		wbcg_edit_start (wbcg, FALSE, TRUE);
 		fd = gnm_expr_get_func_def (expr);
@@ -972,7 +1011,7 @@ dialog_formula_guru (WorkbookControlGUI *wbcg, GnmFunc const *fd)
 	state->dialog = glade_xml_get_widget (state->gui, "formula_guru");
 
 	if (dialog_formula_guru_init (state)) {
-		gnumeric_notice (wbcg_toplevel (wbcg), GTK_MESSAGE_ERROR,
+		go_gtk_notice_dialog (wbcg_toplevel (wbcg), GTK_MESSAGE_ERROR,
 				 _("Could not create the formula guru."));
 		g_free (state);
 		return;
@@ -998,5 +1037,4 @@ dialog_formula_guru (WorkbookControlGUI *wbcg, GnmFunc const *fd)
 	}
 
 	gtk_widget_show_all (state->dialog);
-	return;
 }
