@@ -23,6 +23,8 @@
 #include <gnumeric-config.h>
 #include <glib/gi18n-lib.h>
 #include <gnumeric.h>
+#include <gtk/gtk.h>
+
 #include "dialogs.h"
 #include "help.h"
 
@@ -37,11 +39,6 @@
 #include <style.h>
 #include <gnumeric-gconf.h>
 
-#include <libgnomeprint/gnome-print-job.h>
-#include <libgnomeprint/gnome-print-unit.h>
-#include <libgnomeprintui/gnome-print-paper-selector.h>
-#include <libgnomeprintui/gnome-print-unit-selector.h>
-
 #include <goffice/cut-n-paste/foocanvas/foo-canvas.h>
 #include <goffice/cut-n-paste/foocanvas/foo-canvas-util.h>
 #include <goffice/cut-n-paste/foocanvas/foo-canvas-line.h>
@@ -79,6 +76,8 @@
 
 #define HF_PAGE 1
 
+#if 0
+
 /* FIXME: Now that we have added a header/footer sample
  * preview widget, we should rename the preview widget for the margins
  * to be more specific.
@@ -165,6 +164,16 @@ typedef struct {
 	HFPreviewInfo *pi_footer;
 } PrinterSetupState;
 
+#endif
+
+typedef struct {
+	WorkbookControlGUI  *wbcg;
+	Sheet            *sheet;
+	PrintInformation *pi;
+} PrinterSetupState;
+
+#if 0
+
 typedef struct {
 	PrinterSetupState *state;
 	UnitInfo *target;
@@ -1609,6 +1618,7 @@ do_setup_main_dialog (PrinterSetupState *state)
 
 }
 
+
 static PrinterSetupState *
 printer_setup_state_new (WorkbookControlGUI *wbcg, Sheet *sheet)
 {
@@ -1638,6 +1648,21 @@ printer_setup_state_new (WorkbookControlGUI *wbcg, Sheet *sheet)
 
 	return state;
 }
+#endif
+
+static PrinterSetupState *
+printer_setup_state_new (WorkbookControlGUI *wbcg, Sheet *sheet)
+{
+	PrinterSetupState *state;
+
+	state = g_new0 (PrinterSetupState, 1);
+	state->wbcg  = wbcg;
+	state->sheet = sheet;
+	state->pi    = print_info_dup (sheet->print_info);
+	return state;
+}
+
+#if 0
 
 static void
 do_fetch_page (PrinterSetupState *state)
@@ -1750,27 +1775,59 @@ fetch_settings (PrinterSetupState *state)
 	do_fetch_page_info (state);
 }
 
+#endif
+
+static void
+dialog_printer_setup_done_cb (GtkPageSetup *page_setup,
+			      gpointer data)
+{
+	PrinterSetupState *state = data;
+
+	if (page_setup) {
+		print_info_set_page_setup (state->pi, page_setup);
+		cmd_print_setup (WORKBOOK_CONTROL (state->wbcg),
+				 state->sheet, state->pi);
+	}
+	print_info_free (state->pi);
+	g_free (state);
+}
+
 void
 dialog_printer_setup (WorkbookControlGUI *wbcg, Sheet *sheet)
 {
-	PrinterSetupState *state;
+	PrinterSetupState *state = printer_setup_state_new (wbcg, sheet);
+	GtkPageSetup *page_setup = print_info_get_page_setup (state->pi);
+	
+	gtk_print_run_page_setup_dialog_async
+		(wbcg_toplevel (wbcg),
+		 page_setup,
+		 NULL,
+		 dialog_printer_setup_done_cb,
+		 state);
 
-	/* Only one guru per workbook. */
-	if (wbcg_edit_get_guru (wbcg))
-		return;
+	if (page_setup)
+		g_object_unref (page_setup);
 
-	/* Only pop up one copy per workbook */
-	if (gnumeric_dialog_raise_if_exists (wbcg, PRINTER_SETUP_KEY))
-		return;
 
-	state = printer_setup_state_new (wbcg, sheet);
-	if (!state)
-		return;
+	
+/* 	PrinterSetupState *state; */
 
-	gnumeric_init_help_button (
-		glade_xml_get_widget (state->gui, "help_button"),
-		GNUMERIC_HELP_LINK_PRINTER_SETUP_GENERAL);
-	gnumeric_keyed_dialog (
-		wbcg, GTK_WINDOW (state->dialog), PRINTER_SETUP_KEY);
-	gtk_widget_show (state->dialog);
+/* 	/\* Only one guru per workbook. *\/ */
+/* 	if (wbcg_edit_get_guru (wbcg)) */
+/* 		return; */
+
+/* 	/\* Only pop up one copy per workbook *\/ */
+/* 	if (gnumeric_dialog_raise_if_exists (wbcg, PRINTER_SETUP_KEY)) */
+/* 		return; */
+
+/* 	state = printer_setup_state_new (wbcg, sheet); */
+/* 	if (!state) */
+/* 		return; */
+
+/* 	gnumeric_init_help_button ( */
+/* 		glade_xml_get_widget (state->gui, "help_button"), */
+/* 		GNUMERIC_HELP_LINK_PRINTER_SETUP_GENERAL); */
+/* 	gnumeric_keyed_dialog ( */
+/* 		wbcg, GTK_WINDOW (state->dialog), PRINTER_SETUP_KEY); */
+/* 	gtk_widget_show (state->dialog); */
 }
