@@ -3,7 +3,7 @@
  *   Dialog for entering a search-and-replace query.
  *
  * Author:
- *   Morten Welinder (terra@diku.dk)
+ *   Morten Welinder (terra@gnome.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  */
 
 #include <gnumeric-config.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <gnumeric.h>
 #include "dialogs.h"
 #include "help.h"
@@ -33,9 +33,8 @@
 #include <selection.h>
 
 #include <glade/glade.h>
-#include <gtk/gtktogglebutton.h>
-#include <gtk/gtktable.h>
-#include <gtk/gtkstock.h>
+#include <goffice/gtk/goffice-gtk.h>
+#include <gtk/gtk.h>
 #include <string.h>
 
 #define SEARCH_REPLACE_KEY "search-replace-dialog"
@@ -103,21 +102,26 @@ ok_clicked (G_GNUC_UNUSED GtkWidget *widget, DialogState *dd)
 	char *err;
 	int i;
 	GnmSearchReplaceScope scope;
+	char *search_text, *replace_text;
 
 	i = gnumeric_glade_group_value (gui, scope_group);
 	scope = (i == -1) ? GNM_SRS_SHEET : (GnmSearchReplaceScope)i;
 
+	search_text = g_utf8_normalize (gtk_entry_get_text (dd->search_text), -1, G_NORMALIZE_DEFAULT);
+	replace_text = g_utf8_normalize (gtk_entry_get_text (dd->replace_text), -1, G_NORMALIZE_DEFAULT);
+
 	sr = g_object_new (GNM_SEARCH_REPLACE_TYPE,
 			   "sheet", wb_control_cur_sheet (WORKBOOK_CONTROL (wbcg)),
 			   "scope", scope,
 			   "range-text", gnm_expr_entry_get_text (dd->rangetext),
-			   "search-text", gtk_entry_get_text (dd->search_text),
-			   "replace-text", gtk_entry_get_text (dd->replace_text),
+			   "search-text", search_text,
+			   "replace-text", replace_text,
 			   "is-regexp", gnumeric_glade_group_value (gui, search_type_group) == 1,
 			   "ignore-case", is_checked (gui, "ignore_case"),
 			   "match-words", is_checked (gui, "match_words"),
 			   "preserve-case", is_checked (gui, "preserve_case"),
 			   "query", is_checked (gui, "query"),
+			   "replace-keep-strings", is_checked (gui, "keep_strings"),
 			   "search-strings", is_checked (gui, "search_string"),
 			   "search-other-values", is_checked (gui, "search_other"),
 			   "search-expressions", is_checked (gui, "search_expr"),
@@ -126,8 +130,11 @@ ok_clicked (G_GNUC_UNUSED GtkWidget *widget, DialogState *dd)
 			   "by-row", gnumeric_glade_group_value (gui, direction_group) == 0,
 			   NULL);
 
+	g_free (search_text);
+	g_free (replace_text);
+
 	i = gnumeric_glade_group_value (gui, error_group);
-	sr->error_behaviour = (i == -1) ? SRE_fail : (SearchReplaceError)i;
+	sr->error_behaviour = (i == -1) ? GNM_SRE_FAIL : (GnmSearchReplaceError)i;
 
 	err = gnm_search_replace_verify (sr, TRUE);
 	if (err) {
@@ -163,10 +170,9 @@ cancel_clicked (G_GNUC_UNUSED GtkWidget *widget, DialogState *dd)
 
 
 static void
-dialog_destroy (G_GNUC_UNUSED GtkWidget *widget, DialogState *dd)
+cb_dialog_destroy (DialogState *dd)
 {
-	GladeXML *gui = dd->gui;
-	g_object_unref (G_OBJECT (gui));
+	g_object_unref (G_OBJECT (dd->gui));
 	wbcg_edit_detach_guru (dd->wbcg);
 	memset (dd, 0, sizeof (*dd));
 	g_free (dd);
@@ -262,9 +268,6 @@ dialog_search_replace (WorkbookControlGUI *wbcg,
 	g_signal_connect (G_OBJECT (glade_xml_get_widget (gui, "cancel_button")),
 		"clicked",
 		G_CALLBACK (cancel_clicked), dd);
-	g_signal_connect (G_OBJECT (dialog),
-		"destroy",
-		G_CALLBACK (dialog_destroy), dd);
 	g_signal_connect (GTK_OBJECT (gnm_expr_entry_get_entry (dd->rangetext)),
 		"focus-in-event",
 		G_CALLBACK (range_focused), dd);
@@ -272,10 +275,15 @@ dialog_search_replace (WorkbookControlGUI *wbcg,
 	gnumeric_init_help_button (
 		glade_xml_get_widget (gui, "help_button"),
 		GNUMERIC_HELP_LINK_SEARCH_REPLACE);
+	g_object_set_data_full (G_OBJECT (dialog),
+		"state", dd, (GDestroyNotify) cb_dialog_destroy);
 
 	gtk_widget_show_all (dialog->vbox);
 	gtk_widget_grab_focus (GTK_WIDGET (dd->search_text));
 
+	gnm_dialog_setup_destroy_handlers (dialog, wbcg,
+					   GNM_DIALOG_DESTROY_SHEET_REMOVED);
+
 	wbcg_edit_attach_guru (wbcg, GTK_WIDGET (dialog));
 	non_modal_dialog (wbcg, dialog, SEARCH_REPLACE_KEY);
 }
@@ -290,7 +298,6 @@ dialog_search_replace_query (WorkbookControlGUI *wbcg,
 	GladeXML *gui;
 	GtkDialog *dialog;
 	int res;
-	GtkWindow *toplevel;
 
 	g_return_val_if_fail (wbcg != NULL, 0);
 
@@ -309,44 +316,26 @@ dialog_search_replace_query (WorkbookControlGUI *wbcg,
 			    new_text);
 	set_checked (gui, "qd_query", sr->query);
 
-
-	toplevel = wbcg_toplevel (wbcg);
-	if (GTK_WINDOW (dialog)->transient_parent != toplevel)
-		gtk_window_set_transient_for (GTK_WINDOW (dialog), toplevel);
-
-	gtk_tooltips_set_tip (gtk_tooltips_new (),
-			      gtk_dialog_add_button (dialog, GTK_STOCK_CANCEL,
-						     GTK_RESPONSE_CANCEL),
-			      _("Perform no more replacements"), NULL);
-	gtk_tooltips_set_tip (gtk_tooltips_new (),
-			      gtk_dialog_add_button (dialog, GTK_STOCK_NO,
-						     GTK_RESPONSE_NO),
-			      _("Do not perform this replacement"), NULL);
-	gtk_tooltips_set_tip (gtk_tooltips_new (),
-			      gtk_dialog_add_button (dialog, GTK_STOCK_YES,
-						     GTK_RESPONSE_YES),
-			      _("Perform this replacement"), NULL);
-
+	wbcg_set_transient_for (wbcg, GTK_WINDOW (dialog));
+	go_dialog_guess_alternative_button_order (dialog);
 	gtk_widget_show_all (GTK_WIDGET (dialog));
-	res = gtk_dialog_run (dialog);
 
-	/* Unless cancel is pressed, propagate the query setting back.  */
-	if (res != GTK_RESPONSE_CANCEL && res != GTK_RESPONSE_NONE &&
-	    res != GTK_RESPONSE_DELETE_EVENT)
+	gnm_dialog_setup_destroy_handlers (dialog, wbcg,
+					   GNM_DIALOG_DESTROY_SHEET_REMOVED);
+
+	res = gtk_dialog_run (dialog);
+	switch (res) {
+	case GTK_RESPONSE_YES:
+	case GTK_RESPONSE_NO:
 		sr->query = is_checked (gui, "qd_query");
+		break;
+	default:
+		res = GTK_RESPONSE_CANCEL;
+	}
 
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 
-/*FIXME: rather than recoding the result value we should change the tests down stream */
-	/* Acutally, no.  This needs to be handled as signal on the GnmSearchReplace
-	   object -- MW  */
-
-	if (res == GTK_RESPONSE_YES)
-		return 0;
-	if (res == GTK_RESPONSE_NO)
-		return 1;
-
-	return -1;
+	return res;
 }
 
 /* ------------------------------------------------------------------------- */
