@@ -95,6 +95,8 @@ format_value_common (PangoLayout *layout, GString *str,
 
 	if (format == NULL)
 		format = VALUE_FMT (value);
+	if (format && go_format_is_markup (format))
+		format = NULL;
 
 	/* Use top left corner of an array result.  This will not work for
 	 * ranges because we dont't have a location */
