@@ -24,21 +24,14 @@
 #include "workbook-view.h"
 #include "workbook-priv.h"
 #include "gnumeric-gconf.h"
+#include "application.h"
 
 #include <goffice/gtk/go-charmap-sel.h>
 #include <goffice/app/io-context.h>
 #include <goffice/app/go-doc.h>
 #include <goffice/utils/go-file.h>
 
-#include <gtk/gtkcombobox.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtktable.h>
-#include <gtk/gtkcombo.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtkimage.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtkvbox.h>
-#include <gtk/gtkfilechooserdialog.h>
+#include <gtk/gtk.h>
 #include <glade/glade.h>
 #include <unistd.h>
 #include <errno.h>
@@ -142,6 +135,7 @@ gui_file_read (WorkbookControlGUI *wbcg, char const *uri,
 
 	if (wbv != NULL) {
 		gui_wb_view_show (wbcg, wbv);
+		workbook_update_history (wb_view_get_workbook (wbv));
 		return TRUE;
 	}
 	return FALSE;
@@ -184,10 +178,10 @@ advanced_clicked (GtkButton *advanced, GtkFileChooser *fsel)
 	GtkWidget *extra = g_object_get_data (G_OBJECT (advanced), "extra");
 
 	if (gtk_file_chooser_get_extra_widget (fsel)) {
-		gtk_button_set_label (advanced, _("Simple"));
+		gtk_button_set_label (advanced, _("Advanced"));
 		gtk_file_chooser_set_extra_widget (fsel, NULL);
 	} else {
-		gtk_button_set_label (advanced, _("Advanced"));
+		gtk_button_set_label (advanced, _("Simple"));
 		gtk_file_chooser_set_extra_widget (fsel, extra);
 	}
 }
@@ -272,28 +266,14 @@ gui_file_open (WorkbookControlGUI *wbcg, char const *default_format)
 	/* Filters */
 	{	
 		GtkFileFilter *filter;
-		GList *l;
 
 		filter = gtk_file_filter_new ();
 		gtk_file_filter_set_name (filter, _("All Files"));
 		gtk_file_filter_add_pattern (filter, "*");
 		gtk_file_chooser_add_filter (fsel, filter);
 
-		filter = gtk_file_filter_new ();
+		filter = gnm_app_create_opener_filter ();
 		gtk_file_filter_set_name (filter, _("Spreadsheets"));
-		for (l = openers->next; l; l = l->next) {
-			GOFileOpener const *o = l->data;
-			GSList const *ptr;
-			for (ptr = go_file_opener_get_suffixes	(o); ptr != NULL ; ptr = ptr->next) {
-				char *pattern = g_strconcat ("*.", ptr->data, NULL);
-				gtk_file_filter_add_pattern (filter, pattern);
-				g_free (pattern);
-			}
-			for (ptr = go_file_opener_get_mimes	(o); ptr != NULL ; ptr = ptr->next)
-				gtk_file_filter_add_mime_type (filter, ptr->data);
-
-		}
-
 		gtk_file_chooser_add_filter (fsel, filter);
 		/* Make this filter the default */
 		gtk_file_chooser_set_filter (fsel, filter);
@@ -542,8 +522,10 @@ gui_file_save_as (WorkbookControlGUI *wbcg, WorkbookView *wb_view)
 		gtk_widget_destroy (GTK_WIDGET (fsel));
 		fsel = NULL;
 		success = wb_view_save_as (wb_view, fs, uri, GO_CMD_CONTEXT (wbcg));
-		if (success)
+		if (success) {
 			wbcg->current_saver = fs;
+			workbook_update_history (wb_view_get_workbook (wb_view));
+		}
 	}
 
 	g_free (uri);
@@ -568,6 +550,10 @@ gui_file_save (WorkbookControlGUI *wbcg, WorkbookView *wb_view)
 	wb = wb_view_get_workbook (wb_view);
 	if (wb->file_format_level < FILE_FL_AUTO)
 		return gui_file_save_as (wbcg, wb_view);
-	else
-		return wb_view_save (wb_view, GO_CMD_CONTEXT (wbcg));
+	else {
+		gboolean ok = wb_view_save (wb_view, GO_CMD_CONTEXT (wbcg));
+		if (ok)
+			workbook_update_history (wb);
+		return ok;
+	}
 }
