@@ -27,6 +27,7 @@
 #include "preview-grid-impl.h"
 
 #include "cell.h"
+#include "sheet.h"
 #include "cell-draw.h"
 #include "colrow.h"
 #include "pattern.h"
@@ -52,42 +53,6 @@ enum {
 
 /*****************************************************************************/
 
-static int
-pg_get_row_height (PreviewGrid *pg, int row)
-{
-	PreviewGridClass *klass = PREVIEW_GRID_GET_CLASS (pg);
-	int height;
-
-	g_return_val_if_fail (row >= 0 && row < SHEET_MAX_ROWS, 1);
-	g_return_val_if_fail (klass != NULL, 1);
-
-	if (klass->get_row_height != NULL) {
-		height = (klass->get_row_height) (pg, row);
-		if (height > 0)
-			return height;
-	}
-
-	return pg->defaults.row_height;
-}
-
-static int
-pg_get_col_width (PreviewGrid *pg, int col)
-{
-	PreviewGridClass *klass = PREVIEW_GRID_GET_CLASS (pg);
-	int width;
-
-	g_return_val_if_fail (col >= 0 && col < SHEET_MAX_COLS, 1);
-	g_return_val_if_fail (klass != NULL, 1);
-
-	if (klass->get_col_width != NULL) {
-		width = (klass->get_col_width) (pg, col);
-		if (width > 0)
-			return width;
-	}
-
-	return pg->defaults.col_width;
-}
-
 static GnmStyle *
 pg_get_style (PreviewGrid *pg, int col, int row)
 {
@@ -108,58 +73,31 @@ pg_get_style (PreviewGrid *pg, int col, int row)
 }
 
 static GnmCell *
-pg_construct_cell (PreviewGrid *pg, int col, int row, PangoContext *context)
+pg_fetch_cell (PreviewGrid *pg, int col, int row, PangoContext *context,
+	       GnmStyle const *style)
 {
 	PreviewGridClass *klass = PREVIEW_GRID_GET_CLASS (pg);
-	GnmCell   *cell;
-	GnmStyle *style;
+	GnmCell  *cell;
+	GnmValue *v = NULL;
 
 	g_return_val_if_fail (klass != NULL, NULL);
 	g_return_val_if_fail (pg != NULL, NULL);
 	g_return_val_if_fail (col >= 0 && col < SHEET_MAX_COLS, NULL);
 	g_return_val_if_fail (row >= 0 && row < SHEET_MAX_ROWS, NULL);
 
-	/*
-	 * We are going to manually replicate a cell
-	 * structure here, please remember that this is not
-	 * the equivalent of a real cell.  In particular we g_new
-	 * it and it must be g_free'd.
-	 */
-	cell = g_new0 (GnmCell, 1);
+	if (NULL != klass->get_cell_value)
+		v = (klass->get_cell_value) (pg, col, row);
+	if (NULL == v)
+		v = value_dup (pg->defaults.value);
 
-	cell->col_info = g_new0 (ColRowInfo, 1);
-	cell->row_info = g_new0 (ColRowInfo, 1);
-
-	style = pg_get_style (pg, col, row);
-
-	/*
-	 * Eventually the row_info->pos and col_info->pos
-	 * will go away
-	 */
-	cell->col_info->pos = col;
-	cell->row_info->pos = row;
-	cell->pos.col = col;
-	cell->pos.row = row;
-
-	cell->col_info->margin_a = 2;
-	cell->col_info->margin_b = 2;
-	cell->row_info->margin_a = 0;
-	cell->row_info->margin_b = 0;
-
-	cell->col_info->size_pixels = pg_get_col_width  (pg, col);
-	cell->row_info->size_pixels = pg_get_row_height (pg, row);
-
-	cell->value = NULL;
-	if (klass->get_cell_value != NULL)
-		cell->value = (klass->get_cell_value) (pg, col, row);
-	if (cell->value == NULL)
-		cell->value = value_dup (pg->defaults.value);
-	cell->rendered_value = rendered_value_new (cell, style, TRUE, context, 1.0);
+	cell = sheet_cell_fetch (pg->sheet, col, row);
+	gnm_cell_set_value (cell, v);
+	cell->rendered_value = gnm_rendered_value_new (cell, style,
+		TRUE, context, pg->sheet->last_zoom_factor_used);
 
 	return cell;
 }
 
-
 /**
  * pg_get_row_offset:
  * pg:
@@ -173,12 +111,11 @@ pg_get_row_offset (PreviewGrid *pg, int const y, int *row_origin)
 {
 	int row   = 0;
 	int pixel = 1;
-	int h;
+	int const h = pg->defaults.row_height;
 
 	g_return_val_if_fail (pg != NULL, 0);
 
 	do {
-		h = pg_get_row_height (pg, row);
 		if (y <= (pixel + h) || h == 0) {
 			if (row_origin)
 				*row_origin = pixel;
@@ -210,7 +147,7 @@ pg_get_col_offset (PreviewGrid *pg, int const x, int *col_origin)
 	g_return_val_if_fail (pg != NULL, 0);
 
 	do {
-		w = pg_get_col_width (pg, col);
+		w = pg->defaults.col_width;
 		if (x <= (pixel + w) || w == 0) {
 			if (col_origin)
 				*col_origin = pixel;
@@ -225,20 +162,6 @@ pg_get_col_offset (PreviewGrid *pg, int const x, int *col_origin)
 	return SHEET_MAX_COLS - 1;
 }
 
-static void
-pg_destruct_cell (GnmCell *cell)
-{
-	g_return_if_fail (cell != NULL);
-
-	value_release (cell->value);
-	rendered_value_destroy (cell->rendered_value);
-
-	g_free (cell->col_info);
-	g_free (cell->row_info);
-
-	g_free (cell);
-}
-
 static void
 preview_grid_realize (FooCanvasItem *item)
 {
@@ -304,14 +227,14 @@ preview_grid_draw_background (GdkDrawable *drawable, PreviewGrid const *pg, GnmS
 		/* Fill the entire cell (API excludes far pixel) */
 		gdk_draw_rectangle (drawable, gc, TRUE, x, y, w+1, h+1);
 
-	style_border_draw_diag (mstyle, drawable, x, y, x+w, y+h);
+	gnm_style_border_draw_diag (mstyle, drawable, x, y, x+w, y+h);
 }
 
 #define border_null(b)	((b) == none || (b) == NULL)
 static void
 pg_style_get_row (PreviewGrid *pg, GnmStyleRow *sr)
 {
-	GnmBorder const *top, *bottom, *none = style_border_none ();
+	GnmBorder const *top, *bottom, *none = gnm_style_border_none ();
 	GnmBorder const *left, *right;
 	int const end = sr->end_col, row = sr->row;
 	int col = sr->start_col;
@@ -375,15 +298,16 @@ preview_grid_draw (FooCanvasItem *item, GdkDrawable *drawable,
  	int start_row       = pg_get_row_offset (pg, draw_y - 2, &y);
  	int end_row         = pg_get_row_offset (pg, draw_y + height + 2, NULL);
  	int diff_y    = y;
+	int row_height = pg->defaults.row_height;
 
 	GnmStyleRow sr, next_sr;
 	GnmStyle const **styles;
 	GnmBorder const **borders, **prev_vert;
-	GnmBorder const *none = pg->gridlines ? style_border_none () : NULL;
+	GnmBorder const *none = pg->gridlines ? gnm_style_border_none () : NULL;
 
 	int *colwidths = NULL;
 
-	style_border_none_set_color (style_color_grid ());
+	gnm_style_border_none_set_color (style_color_grid ());
 
 	/*
 	 * allocate a single blob of memory for all 8 arrays of pointers.
@@ -402,7 +326,7 @@ preview_grid_draw (FooCanvasItem *item, GdkDrawable *drawable,
 	colwidths = g_alloca (n * sizeof (int));
 	colwidths -= start_col;
 	for (col = start_col; col <= end_col; col++)
-		colwidths[col] = pg_get_col_width (pg, col);
+		colwidths[col] = pg->defaults.col_width;
 
 	foo_canvas_w2c (item->canvas, diff_x, diff_y, &diff_x, &diff_y);
  	/* Fill entire region with default background (even past far edge) */
@@ -410,8 +334,6 @@ preview_grid_draw (FooCanvasItem *item, GdkDrawable *drawable,
  			    diff_x, diff_y, width, height);
 
 	for (y = diff_y; row <= end_row; row = sr.row = next_sr.row) {
- 		int row_height = pg_get_row_height (pg, row);
-
 		if (++next_sr.row > end_row) {
 			for (col = start_col ; col <= end_col; ++col)
 				next_sr.vertical [col] =
@@ -420,22 +342,22 @@ preview_grid_draw (FooCanvasItem *item, GdkDrawable *drawable,
 			pg_style_get_row (pg, &next_sr);
 
 		for (col = start_col, x = diff_x; col <= end_col; col++) {
- 			GnmCell      *cell  = pg_construct_cell (pg, col, row, context);
 			GnmStyle const *style = sr.styles [col];
+ 			GnmCell const  *cell  = pg_fetch_cell (pg,
+				col, row, context, style);
 
  			preview_grid_draw_background (drawable, pg,
  						      style, col, row, x, y,
  						      colwidths [col], row_height);
 
-			if (!cell_is_empty (cell))
+			if (!gnm_cell_is_empty (cell))
 				cell_draw (cell, pg->gc.cell, drawable,
-					   x, y, -1, -1, -1);
+					   x, y, colwidths [col], row_height, -1);
 
-			pg_destruct_cell (cell);
  			x += colwidths [col];
  		}
 
-		style_borders_row_draw (prev_vert, &sr,
+		gnm_style_borders_row_draw (prev_vert, &sr,
 					drawable, diff_x, y, y+row_height,
 					colwidths, TRUE, 1 /* cheat dir == 1 for now */);
 
@@ -499,7 +421,7 @@ preview_grid_set_property (GObject *obj, guint param_id,
 }
 
 static void
-preview_grid_finalize (GObject *obj)
+preview_grid_dispose (GObject *obj)
 {
 	PreviewGrid *pg = PREVIEW_GRID (obj);
 
@@ -512,7 +434,12 @@ preview_grid_finalize (GObject *obj)
 		pg->defaults.value = NULL;
 	}
 
-	(*G_OBJECT_CLASS (parent_klass)->finalize) (obj);
+	if (pg->sheet) {
+		g_object_unref (pg->sheet);
+		pg->sheet = NULL;
+	}
+
+	G_OBJECT_CLASS (parent_klass)->dispose (obj);
 }
 
 static void
@@ -525,22 +452,25 @@ preview_grid_init (PreviewGrid *pg)
 	item->x2 = 0;
 	item->y2 = 0;
 
-	pg->gridlines      = FALSE;
-	pg->defaults.col_width  = 64;
+	pg->sheet = g_object_new (GNM_SHEET_TYPE, NULL);
+	pg->sheet->index_in_wb = -1;
+	pg->sheet->workbook = NULL;
+	pg->gridlines = FALSE;
+	pg->defaults.col_width = 64;
 	pg->defaults.row_height = 17;
-	pg->defaults.style      = gnm_style_new_default ();
-	pg->defaults.value	= value_new_empty ();
+	pg->defaults.style = gnm_style_new_default ();
+	pg->defaults.value = value_new_empty ();
 }
 
 static void
 preview_grid_class_init (GObjectClass *gobject_klass)
 {
-	FooCanvasItemClass *item_klass = (FooCanvasItemClass *) gobject_klass;
+	FooCanvasItemClass *item_klass = (FooCanvasItemClass *)gobject_klass;
 
 	parent_klass = g_type_class_peek_parent (gobject_klass);
 
-	gobject_klass->finalize     = preview_grid_finalize;
 	gobject_klass->set_property = preview_grid_set_property;
+	gobject_klass->dispose = preview_grid_dispose;
 	g_object_class_install_property (gobject_klass, PREVIEW_GRID_PROP_RENDER_GRIDLINES,
 		g_param_spec_boolean ("render-gridlines", NULL, NULL,
 			FALSE,
