@@ -2,14 +2,11 @@
 #define GNM_PRINT_INFO_H
 
 #include "gnumeric.h"
-
-#ifdef WITH_GNOME_PRINT
-#include <libgnomeprint/gnome-print-config.h>	/* for typedef of GnomePrintConfig */
-#endif
+#include <gtk/gtk.h>
 
 typedef struct {
 	double    points;
-#ifdef WITH_GNOME_PRINT
+#if 0
 	GnomePrintUnit const *desired_display;
 #endif
 } PrintUnit;
@@ -19,7 +16,6 @@ typedef struct {
  */
 typedef struct {
 	PrintUnit top, bottom;     /* see print.c for the definition (these are header/footer) */
-	double	  left, right, header, footer;
 } PrintMargins;
 
 /* Header/Footer definition */
@@ -86,12 +82,11 @@ struct _PrintInformation {
 	PrintHF          *header;
 	PrintHF          *footer;
 
-	unsigned	  n_copies;
 	int		  start_page; /* < 0 implies auto */
-	char		 *paper;
-	char		 *paper_width, *paper_height;
+        int              n_copies;
 
-	char		 *gp_config_str;
+  /* page_setup doubles as a flag whether the defaults are loaded */
+        GtkPageSetup     *page_setup;
 };
 
 typedef enum {
@@ -105,15 +100,14 @@ typedef struct {
 	GnmValue *date_time;
 } HFRenderInfo;
 
-PrintInformation *print_info_new         (void);
-PrintInformation *print_info_dup	 (PrintInformation const *pi);
+PrintInformation *print_info_new         (gboolean load_defaults);
+PrintInformation *print_info_load_defaults (PrintInformation *pi);
+PrintInformation *print_info_dup	 (PrintInformation *pi);
 void              print_info_free        (PrintInformation *pi);
-void              print_info_save        (PrintInformation const *pi);
+void              print_info_save        (PrintInformation *pi);
 
-#ifdef WITH_GNOME_PRINT
-GnomePrintConfig *print_info_make_config (PrintInformation const *pi);
-void		  print_info_load_config (PrintInformation *pi, GnomePrintConfig *config);
-#endif
+GtkPageSetup     *print_info_get_page_setup (PrintInformation const *pi);
+void              print_info_set_page_setup (PrintInformation *pi, GtkPageSetup *page_setup);
 
 PrintHF          *print_hf_new           (char const *left,
 					  char const *middle,
@@ -129,7 +123,7 @@ char             *hf_format_render       (char const *format,
 HFRenderInfo     *hf_render_info_new     (void);
 void              hf_render_info_destroy (HFRenderInfo *hfi);
 
-#ifdef WITH_GNOME_PRINT
+#if 0
 GnomePrintUnit const *unit_name_to_unit    (char const *name);
 double      unit_convert             (double value,
 				      GnomePrintUnit const *source,
@@ -140,12 +134,16 @@ void        print_init               (void);
 void        print_shutdown           (void);
 
 void	    print_info_set_paper     	   (PrintInformation *pi, char const *paper);
-void	    print_info_set_paper_width     (PrintInformation *pi, char const *paper_width);
-void	    print_info_set_paper_height    (PrintInformation *pi, char const *paper_height);
-char const *print_info_get_paper     	   (PrintInformation const *pi);
-char const *print_info_get_paper_width     (PrintInformation const *pi);
-char const *print_info_get_paper_height    (PrintInformation const *pi);
-void        print_info_get_margins   (PrintInformation const *pi,
+void        print_info_set_paper_width_height (PrintInformation *pi,
+					       double paper_width,
+					       double paper_height,
+					       GtkUnit unit);
+char       *print_info_get_paper     	   (PrintInformation *pi);
+/* Note that the string returned by print_info_get_paper must be freed */
+
+double      print_info_get_paper_width     (PrintInformation *pi, GtkUnit unit);
+double      print_info_get_paper_height    (PrintInformation *pi, GtkUnit unit);
+void        print_info_get_margins   (PrintInformation *pi,
 				      double *header, double *footer, double *left, double *right);
 void        print_info_set_margins   (PrintInformation *pi,
 				      double header, double footer, double left, double right);
