@@ -3,7 +3,7 @@
 /*
  * sheet-style.c: storage mechanism for styles and eventually cells.
  *
- * Copyright (C) 2000-2004 Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 2000-2006 Jody Goldberg (jody@gnome.org)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of version 2 of the GNU General Public License as published
@@ -25,7 +25,6 @@
 #include "ranges.h"
 #include "sheet.h"
 #include "expr.h"
-#include "gnm-format.h"
 #include "style.h"
 #include "style-border.h"
 #include "style-color.h"
@@ -33,7 +32,7 @@
 #include "cell.h"
 #include "gutils.h"
 #include <goffice/utils/go-glib-extras.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <string.h>
 
 #ifndef USE_TILE_POOLS
@@ -41,7 +40,7 @@
 #endif
 
 typedef union _CellTile CellTile;
-struct _SheetStyleData {
+struct _GnmSheetStyleData {
 	GHashTable *style_hash;
 	CellTile   *styles;
 	GnmStyle   *default_style;
@@ -87,10 +86,11 @@ sheet_style_find (Sheet const *sheet, GnmStyle *s)
 /* Place holder until I merge in the new styles too */
 static void
 pstyle_set_border (GnmStyle *st, GnmBorder *border,
-		   StyleBorderLocation side)
+		   GnmStyleBorderLocation side)
 {
-	gnm_style_set_border (st, MSTYLE_BORDER_TOP + side,
-			   style_border_ref (border));
+	gnm_style_set_border (st,
+			      GNM_STYLE_BORDER_LOCATION_TO_STYLE_ELEMENT (side),
+			      gnm_style_border_ref (border));
 }
 
 /* Amortize the cost of applying a partial style over a large region
@@ -162,7 +162,7 @@ rstyle_apply (GnmStyle **old, ReplacementStyle *rs)
 		 */
 		s = (GnmStyle *)g_hash_table_lookup (rs->cache, *old);
 		if (s == NULL) {
-			GnmStyle *tmp = gnm_style_merge (*old, rs->pstyle);
+			GnmStyle *tmp = gnm_style_new_merged (*old, rs->pstyle);
 			s = sheet_style_find (rs->sheet, tmp);
 			gnm_style_link (*old);
 			g_hash_table_insert (rs->cache, *old, s);
@@ -327,7 +327,7 @@ cell_tile_style_new (GnmStyle *style, CellTileType t)
 			   break;
 	default : g_return_val_if_fail (FALSE, NULL);
 		return NULL;
-	};
+	}
 #endif
 
 	*((CellTileType *)&(res->type)) = t;
@@ -391,7 +391,7 @@ cell_tile_ptr_matrix_new (CellTile *t)
 		break;
 	}
 	default : ;
-	};
+	}
 
 	return (CellTile *)res;
 }
@@ -435,7 +435,7 @@ cell_tile_matrix_set (CellTile *t, GnmRange const *indic, ReplacementStyle *rs)
 	case TILE_MATRIX :
 	default :
 		break;
-	};
+	}
 
 	if (t->type != TILE_MATRIX)
 		cell_tile_dtor (t);
@@ -498,7 +498,7 @@ sheet_style_init (Sheet *sheet)
 			     "name.  Expect weirdness."));
 	}
 
-	sheet->style_data = g_new (SheetStyleData, 1);
+	sheet->style_data = g_new (GnmSheetStyleData, 1);
 	sheet->style_data->style_hash =
 		g_hash_table_new (gnm_style_hash, (GCompareFunc) gnm_style_equal);
 #warning "FIXME: Allocating a GnmColor here is dubious."
@@ -514,7 +514,7 @@ sheet_style_init (Sheet *sheet)
 	if (sheet->sheet_type == GNM_SHEET_OBJECT) {
 		gnm_style_set_back_color (default_style,
 			style_color_new_i8 (0x50, 0x50, 0x50));
-		kstyle_set_pattern (default_style, 1);
+		gnm_style_set_pattern (default_style, 1);
 	}
 #endif
 	sheet->style_data->default_style =
@@ -536,7 +536,7 @@ static void
 cb_tile_pool_leak (gpointer data, gpointer user)
 {
 	CellTile *tile = data;
-	fprintf (stderr, "Leaking tile at %p.\n", tile);
+	g_printerr ("Leaking tile at %p.\n", tile);
 }
 #endif
 
@@ -597,13 +597,13 @@ sheet_style_shutdown (Sheet *sheet)
 }
 
 /**
- * sheet_style_set_auto_pattern_color
- *
+ * sheet_style_set_auto_pattern_color :
  * @sheet:         The sheet
  * @pattern_color: The color
  *
  * Set the color for rendering auto colored patterns in this sheet.
- */
+ * Absorbs a reference to @pattern_color;
+ **/
 void
 sheet_style_set_auto_pattern_color (Sheet  *sheet, GnmColor *pattern_color)
 {
@@ -623,11 +623,11 @@ sheet_style_set_auto_pattern_color (Sheet  *sheet, GnmColor *pattern_color)
 
 /**
  * sheet_style_get_auto_pattern_color:
- *
  * @sheet: the sheet
  *
+ * Caller receives a reference to the result.
  * Returns the color for rendering auto colored patterns in this sheet.
- */
+ **/
 GnmColor *
 sheet_style_get_auto_pattern_color (Sheet const *sheet)
 {
@@ -644,8 +644,8 @@ sheet_style_get_auto_pattern_color (Sheet const *sheet)
 }
 
 /**
- * This function updates the color of style_border_none when the sheet to be
- * rendered is known. style_border_none tells how to render the
+ * This function updates the color of gnm_style_border_none when the sheet to be
+ * rendered is known. gnm_style_border_none tells how to render the
  * grid. Because the grid color may be different for different sheets, the
  * functions which render the grid call this function first.  The rule for
  * selecting the grid color, which is the same as in Excel, is: - if the
@@ -665,9 +665,9 @@ sheet_style_update_grid_color (Sheet const *sheet)
 		     ? grid_color : sheet_auto);
 
 	/* Do nothing if we already have the right color */
-	if (style_border_none()->color != new_color) {
+	if (gnm_style_border_none()->color != new_color) {
 		style_color_ref (new_color); /* none_set eats the ref */
-		style_border_none_set_color (new_color);
+		gnm_style_border_none_set_color (new_color);
 	}
 	style_color_unref (grid_color);
 	style_color_unref (sheet_auto);
@@ -1043,7 +1043,7 @@ tail_recursion :
  *
  * @sheet :
  * @range :
- * @style
+ * @style : #GnmStyle
  *
  * Change the complete style for a region.
  * This function absorbs a reference to the new @style.
@@ -1063,6 +1063,105 @@ sheet_style_set_range (Sheet *sheet, GnmRange const *range,
 	rstyle_dtor (&rs);
 }
 
+/**
+ * sheet_style_set_col
+ * @sheet :
+ * @col :
+ * @style : #GnmStyle
+ *
+ * NOTE : This is a simple wrapper for now.  When we support col/row styles it
+ * 	will make life easier.
+ *
+ * Change the complete style for a full col.
+ * This function absorbs a reference to the new @style.
+ **/
+void
+sheet_style_set_col (Sheet *sheet, int col, GnmStyle *style)
+{
+	GnmRange r;
+	sheet_style_set_range (sheet, range_init_cols (&r, col, col), style);
+}
+
+/**
+ * sheet_style_apply_col
+ * @sheet :
+ * @col :
+ * @pstyle : #GnmStyle
+ *
+ * NOTE : This is a simple wrapper for now.  When we support col/row styles it
+ * 	will make life easier.
+ *
+ * Apply a partial style to a full col.
+ * The routine absorbs a reference to the partial style.
+ **/
+void
+sheet_style_apply_col (Sheet *sheet, int col, GnmStyle *pstyle)
+{
+	GnmRange r;
+	sheet_style_apply_range (sheet, range_init_cols (&r, col, col), pstyle);
+}
+
+/**
+ * sheet_style_set_row
+ * @sheet :
+ * @row :
+ * @style : #GnmStyle
+ *
+ * NOTE : This is a simple wrapper for now.  When we support col/row styles it
+ * 	will make life easier.
+ *
+ * Change the complete style for a full row.
+ * This function absorbs a reference to the new @style.
+ **/
+void
+sheet_style_set_row (Sheet  *sheet, int row, GnmStyle *style)
+{
+	GnmRange r;
+	sheet_style_set_range (sheet, range_init_rows (&r, row, row), style);
+}
+
+/**
+ * sheet_style_apply_row
+ * @sheet :
+ * @row :
+ * @pstyle : #GnmStyle
+ *
+ * NOTE : This is a simple wrapper for now.  When we support col/row styles it
+ * 	will make life easier.
+ *
+ * Apply a partial style to a full col.
+ * The routine absorbs a reference to the partial style.
+ **/
+void
+sheet_style_apply_row (Sheet  *sheet, int row, GnmStyle *pstyle)
+{
+	GnmRange r;
+	sheet_style_apply_range (sheet, range_init_rows (&r, row, row), pstyle);
+}
+
+/**
+ * sheet_style_apply_pos :
+ * @sheet :
+ * @col   :
+ * @row   :
+ * @pstyle : #GnmStyle
+ *
+ * Apply a partial style to a single cell
+ * This function absorbs a reference to the the new @style.
+ **/
+void
+sheet_style_apply_pos (Sheet *sheet, int col, int row,
+		       GnmStyle *pstyle)
+{
+	ReplacementStyle rs;
+
+	g_return_if_fail (IS_SHEET (sheet));
+
+	cell_tile_apply_pos (&sheet->style_data->styles,
+			     TILE_TOP_LEVEL, col, row,
+			     rstyle_ctor (&rs, NULL, pstyle, sheet));
+	rstyle_dtor (&rs);
+}
 /**
  * sheet_style_set_pos :
  * @sheet :
@@ -1105,14 +1204,14 @@ sheet_style_default (Sheet const *sheet)
 
 /**
  * sheet_style_get :
- *
- * @sheet :
+ * @sheet : #Sheet
  * @col   :
  * @row   :
  *
  * Find the fully qualified style applicable to the specified cellpos.
- */
-GnmStyle *
+ * Does _not_ add a reference.
+ **/
+GnmStyle const *
 sheet_style_get (Sheet const *sheet, int col, int row)
 {
 	int width = TILE_SIZE_COL*TILE_SIZE_COL*TILE_SIZE_COL;
@@ -1147,7 +1246,7 @@ tail_recursion :
 
 	default :
 		break;
-	};
+	}
 
 	g_warning ("Adaptive Quad Tree corruption !");
 	return NULL;
@@ -1158,7 +1257,7 @@ tail_recursion :
 static inline void
 style_row (GnmStyle *style, int start_col, int end_col, GnmStyleRow *sr, gboolean accept_conditions)
 {
-	GnmBorder const *top, *bottom, *none = style_border_none ();
+	GnmBorder const *top, *bottom, *none = gnm_style_border_none ();
 	GnmBorder const *left, *right, *v;
 	int const end = MIN (end_col, sr->end_col);
 	int i = MAX (start_col, sr->start_col);
@@ -1283,7 +1382,7 @@ sheet_style_get_row (Sheet const *sheet, GnmStyleRow *sr)
 	g_return_if_fail (sr->bottom != NULL);
 
 	sr->sheet = sheet;
-	sr->vertical [sr->start_col] = style_border_none ();
+	sr->vertical [sr->start_col] = gnm_style_border_none ();
 	get_style_row (sheet->style_data->styles, TILE_TOP_LEVEL, 0, 0, sr);
 }
 
@@ -1299,7 +1398,7 @@ style_row_init (GnmBorder const * * *prev_vert,
 		int start_col, int end_col, gpointer mem, gboolean hide_grid)
 {
 	int n, col;
-	GnmBorder const *none = hide_grid ? NULL : style_border_none ();
+	GnmBorder const *none = hide_grid ? NULL : gnm_style_border_none ();
 
 	/* alias the arrays for easy access so that array [col] is valid
 	 * for all elements start_col-1 .. end_col+1 inclusive.
@@ -1354,7 +1453,7 @@ sheet_style_apply_range (Sheet *sheet, GnmRange const *range, GnmStyle *pstyle)
 
 static void
 apply_border (Sheet *sheet, GnmRange const *r,
-	      StyleBorderLocation side,
+	      GnmStyleBorderLocation side,
 	      GnmBorder *border)
 {
 	GnmStyle *pstyle = gnm_style_new ();
@@ -1385,118 +1484,118 @@ sheet_style_apply_border (Sheet       *sheet,
 	if (borders == NULL)
 		return;
 
-	if (borders [STYLE_BORDER_TOP]) {
+	if (borders [GNM_STYLE_BORDER_TOP]) {
 		/* 1.1 top inner */
 		GnmRange r = *range;
 		r.end.row = r.start.row;
-		apply_border (sheet, &r, STYLE_BORDER_TOP,
-			      borders [STYLE_BORDER_TOP]);
+		apply_border (sheet, &r, GNM_STYLE_BORDER_TOP,
+			      borders [GNM_STYLE_BORDER_TOP]);
 
 		/* 1.2 top outer */
 		r.start.row--;
 		if (r.start.row >= 0) {
 			r.end.row = r.start.row;
-			apply_border (sheet, &r, STYLE_BORDER_BOTTOM,
-				      style_border_none ());
+			apply_border (sheet, &r, GNM_STYLE_BORDER_BOTTOM,
+				      gnm_style_border_none ());
 		}
 	}
 
-	if (borders [STYLE_BORDER_BOTTOM]) {
+	if (borders [GNM_STYLE_BORDER_BOTTOM]) {
 		/* 2.1 bottom inner */
 		GnmRange r = *range;
 		r.start.row = r.end.row;
-		apply_border (sheet, &r, STYLE_BORDER_BOTTOM,
-			      borders [STYLE_BORDER_BOTTOM]);
+		apply_border (sheet, &r, GNM_STYLE_BORDER_BOTTOM,
+			      borders [GNM_STYLE_BORDER_BOTTOM]);
 
 		/* 2.2 bottom outer */
 		r.end.row++;
 		if (r.end.row < (SHEET_MAX_ROWS-1)) {
 			r.start.row = r.end.row;
-			apply_border (sheet, &r, STYLE_BORDER_TOP,
-				      style_border_none ());
+			apply_border (sheet, &r, GNM_STYLE_BORDER_TOP,
+				      gnm_style_border_none ());
 		}
 	}
 
-	if (borders [STYLE_BORDER_LEFT]) {
+	if (borders [GNM_STYLE_BORDER_LEFT]) {
 		/* 3.1 left inner */
 		GnmRange r = *range;
 		r.end.col = r.start.col;
-		apply_border (sheet, &r, STYLE_BORDER_LEFT,
-			      borders [STYLE_BORDER_LEFT]);
+		apply_border (sheet, &r, GNM_STYLE_BORDER_LEFT,
+			      borders [GNM_STYLE_BORDER_LEFT]);
 
 		/* 3.2 left outer */
 		r.start.col--;
 		if (r.start.col >= 0) {
 			r.end.col = r.start.col;
-			apply_border (sheet, &r, STYLE_BORDER_RIGHT,
-				      style_border_none ());
+			apply_border (sheet, &r, GNM_STYLE_BORDER_RIGHT,
+				      gnm_style_border_none ());
 		}
 	}
 
-	if (borders [STYLE_BORDER_RIGHT]) {
+	if (borders [GNM_STYLE_BORDER_RIGHT]) {
 		/* 4.1 right inner */
 		GnmRange r = *range;
 		r.start.col = r.end.col;
-		apply_border (sheet, &r, STYLE_BORDER_RIGHT,
-			      borders [STYLE_BORDER_RIGHT]);
+		apply_border (sheet, &r, GNM_STYLE_BORDER_RIGHT,
+			      borders [GNM_STYLE_BORDER_RIGHT]);
 
 		/* 4.2 right outer */
 		r.end.col++;
 		if (r.end.col < (SHEET_MAX_COLS-1)) {
 			r.start.col = r.end.col;
-			apply_border (sheet, &r, STYLE_BORDER_LEFT,
-				      style_border_none ());
+			apply_border (sheet, &r, GNM_STYLE_BORDER_LEFT,
+				      gnm_style_border_none ());
 		}
 	}
 
 	/* Interiors horizontal : prefer top */
-	if (borders [STYLE_BORDER_HORIZ] != NULL) {
+	if (borders [GNM_STYLE_BORDER_HORIZ] != NULL) {
 		/* 5.1 horizontal interior top */
 		if (range->start.row != range->end.row) {
 			GnmRange r = *range;
 			++r.start.row;
-			apply_border (sheet, &r, STYLE_BORDER_TOP,
-				      borders [STYLE_BORDER_HORIZ]);
+			apply_border (sheet, &r, GNM_STYLE_BORDER_TOP,
+				      borders [GNM_STYLE_BORDER_HORIZ]);
 		}
 		/* 5.2 interior bottom */
 		if (range->start.row != range->end.row) {
 			GnmRange r = *range;
 			--r.end.row;
-			apply_border (sheet, &r, STYLE_BORDER_BOTTOM,
-				      style_border_none ());
+			apply_border (sheet, &r, GNM_STYLE_BORDER_BOTTOM,
+				      gnm_style_border_none ());
 		}
 	}
 
 	/* Interiors vertical : prefer left */
-	if (borders [STYLE_BORDER_VERT] != NULL) {
+	if (borders [GNM_STYLE_BORDER_VERT] != NULL) {
 		/* 6.1 vertical interior left */
 		if (range->start.col != range->end.col) {
 			GnmRange r = *range;
 			++r.start.col;
-			apply_border (sheet, &r, STYLE_BORDER_LEFT,
-				      borders [STYLE_BORDER_VERT]);
+			apply_border (sheet, &r, GNM_STYLE_BORDER_LEFT,
+				      borders [GNM_STYLE_BORDER_VERT]);
 		}
 
 		/* 6.2 The vertical interior right */
 		if (range->start.col != range->end.col) {
 			GnmRange r = *range;
 			--r.end.col;
-			apply_border (sheet, &r, STYLE_BORDER_RIGHT,
-				      style_border_none ());
+			apply_border (sheet, &r, GNM_STYLE_BORDER_RIGHT,
+				      gnm_style_border_none ());
 		}
 	}
 
 	/* 7. Diagonals (apply both in one pass) */
-	if (borders [STYLE_BORDER_DIAG] != NULL) {
+	if (borders [GNM_STYLE_BORDER_DIAG] != NULL) {
 		pstyle = gnm_style_new ();
-		pstyle_set_border (pstyle, borders [STYLE_BORDER_DIAG],
-				   STYLE_BORDER_DIAG);
+		pstyle_set_border (pstyle, borders [GNM_STYLE_BORDER_DIAG],
+				   GNM_STYLE_BORDER_DIAG);
 	}
-	if (borders [STYLE_BORDER_REV_DIAG]) {
+	if (borders [GNM_STYLE_BORDER_REV_DIAG]) {
 		if (pstyle == NULL)
 			pstyle = gnm_style_new ();
-		pstyle_set_border (pstyle, borders [STYLE_BORDER_REV_DIAG],
-				   STYLE_BORDER_REV_DIAG);
+		pstyle_set_border (pstyle, borders [GNM_STYLE_BORDER_REV_DIAG],
+				   GNM_STYLE_BORDER_REV_DIAG);
 	}
 	if (pstyle != NULL)
 		sheet_style_apply_range (sheet, range, pstyle);
@@ -1519,40 +1618,40 @@ cb_find_conflicts (GnmStyle *style,
 
 static void
 border_mask_internal (gboolean *known, GnmBorder **borders,
-		      GnmBorder const *b, StyleBorderLocation l)
+		      GnmBorder const *b, GnmStyleBorderLocation l)
 {
 	if (!known [l]) {
 		known [l] = TRUE;
 		borders [l] = (GnmBorder *)b;
-		style_border_ref (borders [l]);
+		gnm_style_border_ref (borders [l]);
 	} else if (borders [l] != b && borders [l] != NULL) {
-		style_border_unref (borders [l]);
+		gnm_style_border_unref (borders [l]);
 		borders [l] = NULL;
 	}
 }
 
 static void
 border_mask (gboolean *known, GnmBorder **borders,
-	     GnmBorder const *b, StyleBorderLocation l)
+	     GnmBorder const *b, GnmStyleBorderLocation l)
 {
 	if (b == NULL)
-		b = style_border_none ();
+		b = gnm_style_border_none ();
 	border_mask_internal (known, borders, b, l);
 }
 
 static void
 border_mask_vec (gboolean *known, GnmBorder **borders,
 		 GnmBorder const * const *vec, int first, int last,
-		 StyleBorderLocation l)
+		 GnmStyleBorderLocation l)
 {
 	GnmBorder const *b = vec [first];
 
 	if (b == NULL)
-		b = style_border_none ();
+		b = gnm_style_border_none ();
 	while (first++ < last) {
 		GnmBorder const *tmp = vec [first];
 		if (tmp == NULL)
-			tmp = style_border_none ();
+			tmp = gnm_style_border_none ();
 		if (b != tmp) {
 			b = NULL;
 			break;
@@ -1578,9 +1677,9 @@ sheet_style_find_conflicts (Sheet const *sheet, GnmRange const *r,
 {
 	int n, col, row, start_col, end_col;
 	GnmStyleRow sr;
-	StyleBorderLocation i;
-	gboolean known [STYLE_BORDER_EDGE_MAX];
-	GnmBorder const *none = style_border_none ();
+	GnmStyleBorderLocation i;
+	gboolean known [GNM_STYLE_BORDER_EDGE_MAX];
+	GnmBorder const *none = gnm_style_border_none ();
 	FindConflicts user;
 
 	g_return_val_if_fail (IS_SHEET (sheet), 0);
@@ -1592,12 +1691,12 @@ sheet_style_find_conflicts (Sheet const *sheet, GnmRange const *r,
 	if (*style == NULL) {
 		GnmStyle const *tmp = sheet_style_get (sheet, r->start.col, r->start.row);
 		*style = gnm_style_dup (tmp);
-		for (i = STYLE_BORDER_TOP ; i < STYLE_BORDER_EDGE_MAX ; i++) {
+		for (i = GNM_STYLE_BORDER_TOP ; i < GNM_STYLE_BORDER_EDGE_MAX ; i++) {
 			known [i] = FALSE;
-			borders [i] = style_border_ref ((GnmBorder *)none);
+			borders [i] = gnm_style_border_ref ((GnmBorder *)none);
 		}
 	} else {
-		for (i = STYLE_BORDER_TOP ; i < STYLE_BORDER_EDGE_MAX ; i++)
+		for (i = GNM_STYLE_BORDER_TOP ; i < GNM_STYLE_BORDER_EDGE_MAX ; i++)
 			known [i] = TRUE;
 	}
 
@@ -1608,12 +1707,14 @@ sheet_style_find_conflicts (Sheet const *sheet, GnmRange const *r,
 		      (ForeachTileFunc)cb_find_conflicts, &user);
 
 	/* copy over the diagonals */
-	for (i = STYLE_BORDER_REV_DIAG ; i <= STYLE_BORDER_DIAG ; i++)
-		if (!(user.conflicts & (1 << (MSTYLE_BORDER_TOP+i))))
-			borders [i] = style_border_ref (
-				gnm_style_get_border (*style, MSTYLE_BORDER_TOP+i));
+	for (i = GNM_STYLE_BORDER_REV_DIAG ; i <= GNM_STYLE_BORDER_DIAG ; i++) {
+		GnmStyleElement se = GNM_STYLE_BORDER_LOCATION_TO_STYLE_ELEMENT (i);
+		if (user.conflicts & (1 << se))
+			borders[i] = NULL;
 		else
-			borders [i] = NULL;
+			borders[i] = gnm_style_border_ref (
+				gnm_style_get_border (*style, se));
+	}
 
 	start_col = r->start.col;
 	if (r->start.col > 0)
@@ -1661,16 +1762,16 @@ sheet_style_find_conflicts (Sheet const *sheet, GnmRange const *r,
 		sheet_style_get_row (sheet, &sr);
 
 		border_mask (known, borders, sr.vertical [r->start.col],
-			     STYLE_BORDER_LEFT);
+			     GNM_STYLE_BORDER_LEFT);
 		border_mask (known, borders, sr.vertical [r->end.col+1],
-			     STYLE_BORDER_RIGHT);
+			     GNM_STYLE_BORDER_RIGHT);
 		border_mask_vec (known, borders, sr.top,
 				 r->start.col, r->end.col, (row == r->start.row)
-				 ? STYLE_BORDER_TOP : STYLE_BORDER_HORIZ);
+				 ? GNM_STYLE_BORDER_TOP : GNM_STYLE_BORDER_HORIZ);
 		if (r->start.col != r->end.col)
 			border_mask_vec (known, borders, sr.vertical,
 					 r->start.col+1, r->end.col,
-					 STYLE_BORDER_VERT);
+					 GNM_STYLE_BORDER_VERT);
 
 		roller = sr.top; sr.top = sr.bottom; sr.bottom = roller;
 	}
@@ -1681,7 +1782,7 @@ sheet_style_find_conflicts (Sheet const *sheet, GnmRange const *r,
 		sheet_style_get_row (sheet, &sr);
 	}
 	border_mask_vec (known, borders, sr.top, r->start.col, r->end.col,
-			 STYLE_BORDER_BOTTOM);
+			 GNM_STYLE_BORDER_BOTTOM);
 
 	return user.conflicts;
 }
@@ -1864,14 +1965,14 @@ sheet_style_get_extent (Sheet const *sheet, GnmRange *res,
 /****************************************************************************/
 
 static GnmStyleRegion *
-style_region_new (GnmRange const *range, GnmStyle *mstyle)
+style_region_new (GnmRange const *range, GnmStyle *style)
 {
 	GnmStyleRegion *sr;
 
 	sr = g_new (GnmStyleRegion, 1);
 	sr->range = *range;
-	sr->style = mstyle;
-	gnm_style_ref (mstyle);
+	sr->style = style;
+	gnm_style_ref (style);
 
 	return sr;
 }
@@ -2022,12 +2123,12 @@ sheet_style_get_list (Sheet const *sheet, GnmRange const *r)
 		      TILE_TOP_LEVEL, 0, 0, r,
 		      cb_style_list_add_node, &mi);
 #ifdef DEBUG_STYLE_LIST
-	fprintf(stderr, "=========\n");
+	g_printerr ("=========\n");
 #endif
 	g_hash_table_foreach_remove (mi.cache, cb_hash_merge_horiz, &mi);
 	g_hash_table_foreach_remove (mi.cache, cb_hash_to_list, &res);
 #ifdef DEBUG_STYLE_LIST
-	fprintf(stderr, "=========\n");
+	g_printerr ("=========\n");
 #endif
 	g_hash_table_destroy (mi.cache);
 
@@ -2072,12 +2173,12 @@ sheet_style_collect_conditions (Sheet const *sheet, GnmRange const *r)
 		      TILE_TOP_LEVEL, 0, 0, r,
 		      cb_style_list_add_conditions, &mi);
 #ifdef DEBUG_STYLE_LIST
-	fprintf(stderr, "=========\n");
+	g_printerr ("=========\n");
 #endif
 	g_hash_table_foreach_remove (mi.cache, cb_hash_merge_horiz, &mi);
 	g_hash_table_foreach_remove (mi.cache, cb_hash_to_list, &res);
 #ifdef DEBUG_STYLE_LIST
-	fprintf(stderr, "=========\n");
+	g_printerr ("=========\n");
 #endif
 	g_hash_table_destroy (mi.cache);
 
@@ -2124,12 +2225,12 @@ sheet_style_collect_validations (Sheet const *sheet, GnmRange const *r)
 		      TILE_TOP_LEVEL, 0, 0, r,
 		      cb_style_list_add_validation, &mi);
 #ifdef DEBUG_STYLE_LIST
-	fprintf(stderr, "=========\n");
+	g_printerr ("=========\n");
 #endif
 	g_hash_table_foreach_remove (mi.cache, cb_hash_merge_horiz, &mi);
 	g_hash_table_foreach_remove (mi.cache, cb_hash_to_list, &res);
 #ifdef DEBUG_STYLE_LIST
-	fprintf(stderr, "=========\n");
+	g_printerr ("=========\n");
 #endif
 	g_hash_table_destroy (mi.cache);
 
@@ -2147,11 +2248,11 @@ sheet_style_collect_validations (Sheet const *sheet, GnmRange const *r)
  * Applies a list of styles to the sheet with the supplied offset.  Optionally
  * transposing the ranges
  **/
-SpanCalcFlags
+GnmSpanCalcFlags
 sheet_style_set_list (Sheet *sheet, GnmCellPos const *corner,
 		      gboolean transpose, GnmStyleList const *list)
 {
-	SpanCalcFlags spanflags = SPANCALC_SIMPLE;
+	GnmSpanCalcFlags spanflags = GNM_SPANCALC_SIMPLE;
 	GnmStyleList const *l;
 
 	g_return_val_if_fail (IS_SHEET (sheet), spanflags);
@@ -2322,7 +2423,7 @@ cb_validate (GnmStyle *style,
 			   corner_col, corner_row,
 			   corner_col+width -1, corner_row+height-1);
 		g_warning ("%s!%s", sheet->name_unquoted,
-			   range_name (&r));
+			   range_as_string (&r));
 	}
 }
 
