@@ -61,10 +61,7 @@
 #include "gui-file.h"
 #include "gnumeric-gconf.h"
 #include "expr.h"
-
-#ifdef WITH_GNOME_PRINT
 #include "print.h"
-#endif
 
 #include <goffice/app/io-context.h>
 #include <goffice/graph/gog-guru.h>
@@ -98,9 +95,7 @@ static GNM_ACTION_DEF (cb_file_sendto)	{
 	wb_view_sendto (wb_control_view (WORKBOOK_CONTROL (wbcg)), GO_CMD_CONTEXT (wbcg)); }
 static GNM_ACTION_DEF (cb_file_page_setup)
 { 
-#ifdef WITH_GNOME_PRINT
 	dialog_printer_setup (wbcg, wbcg_cur_sheet (wbcg));
-#endif
 }
 
 static GNM_ACTION_DEF (cb_file_print_area_set)
@@ -152,16 +147,12 @@ static GNM_ACTION_DEF (cb_file_print_area_show)
 
 static GNM_ACTION_DEF (cb_file_print)
 {
-#ifdef WITH_GNOME_PRINT
 	gnm_print_sheet (wbcg, wbcg_cur_sheet (wbcg), FALSE, PRINT_ACTIVE_SHEET);
-#endif
 }
 
 static GNM_ACTION_DEF (cb_file_print_preview)
 {
-#ifdef WITH_GNOME_PRINT
 	gnm_print_sheet (wbcg, wbcg_cur_sheet (wbcg), TRUE, PRINT_ACTIVE_SHEET);
-#endif
 }
 
 static GNM_ACTION_DEF (cb_doc_meta_data)	{ dialog_doc_metadata_new (wbcg); }
@@ -914,7 +905,7 @@ static GNM_ACTION_DEF (cb_help_docs)
 
 #ifndef G_OS_WIN32
 	argv[0] = (char *)"yelp";
-	argv[1] = (char *)"help:gnumeric";
+	argv[1] = (char *)"ghelp:gnumeric";
 	g_spawn_async (NULL, argv, NULL, G_SPAWN_SEARCH_PATH,
 		NULL, NULL, NULL, &error);
 #else
@@ -1037,7 +1028,7 @@ sort_by_rows (WorkbookControlGUI *wbcg, gboolean descending)
 
 	numclause = range_width (sel);
 	clause = g_new0 (GnmSortClause, numclause);
-	for (i=0; i < numclause; i++) {
+	for (i = 0; i < numclause; i++) {
 		clause[i].offset = i;
 		clause[i].asc = descending;
 		clause[i].cs = gnm_app_prefs->sort_default_by_case;
@@ -1049,6 +1040,7 @@ sort_by_rows (WorkbookControlGUI *wbcg, gboolean descending)
 	data->range = sel;
 	data->num_clause = numclause;
 	data->clauses = clause;
+	data->locale = NULL;
 
 	data->retain_formats = gnm_app_prefs->sort_default_retain_formats;
 
@@ -1294,53 +1286,68 @@ static void cb_font_superscript (GtkToggleAction *act, WorkbookControlGUI *wbcg)
 
 static void
 apply_number_format (WorkbookControlGUI *wbcg,
-		     char const *translated_format, char const *descriptor)
+		     GOFormat const *format,
+		     char const *descriptor)
 {
 	GnmStyle *mstyle = gnm_style_new ();
-	gnm_style_set_format_text (mstyle, translated_format);
+	gnm_style_set_format (mstyle, format);
 	cmd_selection_format (WORKBOOK_CONTROL (wbcg), mstyle, NULL, descriptor);
 }
 
 static GNM_ACTION_DEF (cb_format_as_general)
 {
 	apply_number_format (wbcg,
-		go_format_builtins [GO_FORMAT_GENERAL][0], _("Format as General"));
+			     go_format_general (),
+			     _("Format as General"));
 }
+
 static GNM_ACTION_DEF (cb_format_as_number)
 {
-	apply_number_format (wbcg,
-		go_format_builtins [GO_FORMAT_NUMBER][0], _("Format as Number"));
+	GOFormat *fmt = go_format_new_from_XL ("0");
+	apply_number_format (wbcg, fmt, _("Format as Number"));
+	go_format_unref (fmt);
 }
+
 static GNM_ACTION_DEF (cb_format_as_currency)
 {
-	apply_number_format (wbcg,
-		go_format_builtins [GO_FORMAT_CURRENCY][0], _("Format as Currency"));
+	GOFormat *fmt = go_format_new_from_XL (go_format_builtins[GO_FORMAT_CURRENCY][0]);
+	apply_number_format (wbcg, fmt, _("Format as Currency"));
+	go_format_unref (fmt);
 }
+
 static GNM_ACTION_DEF (cb_format_as_accounting)
 {
-	apply_number_format (wbcg,
-		go_format_builtins[GO_FORMAT_ACCOUNTING][2], _("Format as Accounting"));
+	GOFormat *fmt = go_format_new_from_XL (go_format_builtins[GO_FORMAT_ACCOUNTING][2]);
+	apply_number_format (wbcg, fmt, _("Format as Accounting"));
+	go_format_unref (fmt);
 }
 
 static GNM_ACTION_DEF (cb_format_as_percentage)
 {
-	apply_number_format (wbcg,
-		go_format_builtins [GO_FORMAT_PERCENTAGE][0], _("Format as Percentage"));
+	GOFormat *fmt = go_format_new_from_XL ("0%");
+	apply_number_format (wbcg, fmt, _("Format as Percentage"));
+	go_format_unref (fmt);
 }
+
 static GNM_ACTION_DEF (cb_format_as_scientific)
 {
-	apply_number_format (wbcg,
-		go_format_builtins [GO_FORMAT_SCIENTIFIC][0], _("Format as Scientific"));
+	GOFormat *fmt = go_format_new_from_XL ("0.00E+00");
+	apply_number_format (wbcg, fmt, _("Format as Percentage"));
+	go_format_unref (fmt);
 }
+
 static GNM_ACTION_DEF (cb_format_as_time)
 {
 	apply_number_format (wbcg,
-		go_format_builtins [GO_FORMAT_DATE][0], _("Format as Date"));
+			     go_format_default_time (),
+			     _("Format as Time"));
 }
+
 static GNM_ACTION_DEF (cb_format_as_date)
 {
 	apply_number_format (wbcg,
-		go_format_builtins [GO_FORMAT_TIME][0], _("Format as Time"));
+			     go_format_default_date (),
+			     _("Format as Date"));
 }
 
 /* Adds borders to all the selected regions on the sheet.
