@@ -75,6 +75,17 @@ cb_saver_finalize (Workbook *wb, GOFileSaver *saver)
 	wb->file_saver = NULL;
 }
 
+void
+workbook_update_history (Workbook *wb)
+{
+	if (wb->doc.uri && wb->file_format_level >= FILE_FL_MANUAL_REMEMBER) {
+		const char *mimetype = wb->file_saver
+			? go_file_saver_get_mime_type (wb->file_saver)
+			: NULL;
+		gnm_app_history_add (wb->doc.uri, mimetype);
+	}
+}
+
 static void
 workbook_dispose (GObject *wb_object)
 {
@@ -127,10 +138,6 @@ workbook_dispose (GObject *wb_object)
 			g_warning ("Unexpected left over views");
 	}
 
-	if (wb->doc.uri &&
-	    wb->file_format_level >= FILE_FL_MANUAL_REMEMBER)
-		gnm_app_history_add (wb->doc.uri);
-
 	workbook_parent_class->dispose (wb_object);
 }
 
